%{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "lists.h"
  
#define TAILLEPILE 1024
#define TAILLEBUFF 1024

  extern void yyerror(const char *str);
  extern FILE* yyin;
  extern int yylineno;

  List declarations=NULL;
  int instrno=0;

  char buff[TAILLEBUFF];

  int label_no=0; /* Compteur des etiquetes */
  int label_pile[TAILLEPILE]; /* Pile des tiquetes */
  int i=0 ; /* Index de pile */
  
  void build(char *);
  void branch(int);
  void load(int);
  void add(void);
  void ret(void);

%}

/* Symboles terminaux */

%union {
  char* str;
}

%token PLUS
%token LPAR RPAR
%token IF THEN ELSE
%token LET EQ IN
%token <str> ID 
%token <str> CONST

%start S


%%
 /*Begin solution*/

S:
b_expression 

b_expression:
e_expression {ret();}
| LPAR b_expression RPAR 
| LET idinlet EQ e_expression 
{declarations=list_add(buff,declarations);}
IN b_expression 
| IF e_expression THEN {
  label_pile[i]= label_no++;
  branch(label_pile[i]);
  i++;
  if(i > TAILLEPILE) 
    {fprintf(stderr,"Debordement de la pile des etiquetues\n");
      exit(EXIT_FAILURE);}
} 
 b_expression ELSE {
   i--;
   printf("l%d:\n",label_pile[i]);
 } b_expression

e_expression:
ID {load(list_index($1,declarations));}
| CONST {build($1);}
| e_expression PLUS  e_expression {add();}
| LPAR e_expression RPAR 



idinlet:
ID {strncpy(buff,$1,TAILLEBUFF-1);}

/*End solution*/
%%

void yyerror(char const *str)
{
  fprintf(stderr,"Ligne %d : %s\n", yylineno, str); 
  exit(EXIT_FAILURE);
}


int 
main(int argc, char* argv[])
{
  if ( argc > 1 ) {
    if((yyin = fopen( argv[1], "r" ))==NULL)
      exit(EXIT_FAILURE);
    
    yyparse();
    fclose(yyin);
  }
  else exit(EXIT_FAILURE);

  exit(EXIT_SUCCESS);
}


void
build(char * s){
  printf("\t%d : \tbuild \t%s\n",++instrno,s);
}

void
branch(int label){
  printf("\t%d : \tbranch \tl%d\n",++instrno,label);
}

void
load(int v){
  printf("\t%d : \tload \t%d\n",++instrno,v);
}

void
add(void){
  printf("\t%d : \tadd\n",++instrno);
}

void
ret(void){
  printf("\t%d : \treturn\n",++instrno);
}



